@@ -253,7 +253,9 @@ class Data:
         self.port = port or os.getenv('WEAVEIO_PORT', 7687)
         self.password = password or os.getenv('WEAVEIO_PASSWORD', 'weavepassword')
         self.user = user or os.getenv('WEAVEIO_USER', 'weaveuser')
-        self.rootdir = Path(rootdir or os.getenv('WEAVEIO_ROOTDIR', '/beegfs/car/weave/weaveio/'))
+        self.rootdir = Path(rootdir or os.getenv('WEAVEIO_ROOTDIR'))
+        if self.rootdir is None:
+            raise ValueError(f"You must specify rootdir as an environment variable or as an argument to Data")
         self.write_allowed = False
         self.query = Query(self)
         self.rowparser = RowParser(self.rootdir)
