@@ -10,20 +10,23 @@ class L1(Hierarchy):
     is_template = True
 
 
+class L1Spectrum(Spectrum1D, L1):
+    is_template = True
+    children = Spectrum1D.children + [Optional('self', idname='adjunct', one2one=True)]
+    products = ['flux', 'ivar', 'sensfunc']
+    factors = Spectrum.factors + ['nspec', 'snr'] + MeanFlux.as_factors('g', 'r', 'i', 'gg', 'bp', 'rp', prefix='mean_flux_')
+    parents = [FibreTarget]
+    indexes = ['nspec']
+
+
 class NoSS(Spectrum1D):
     plural_name = 'nosses'
     singular_name = 'noss'
     products = ['flux', 'ivar']
-    children = [Optional('self', idname='adjunct')]
-    indexes = [None]
-
-
-class L1Spectrum(Spectrum1D, L1):
-    is_template = True
-    children = Spectrum1D.children + [Optional('self', idname='adjunct'), OneOf(NoSS, one2one=True)]
-    products = ['flux', 'ivar', 'sensfunc']
-    factors = Spectrum.factors + ['nspec', 'snr'] + MeanFlux.as_factors('g', 'r', 'i', 'gg', 'bp', 'rp', prefix='mean_flux_')
-    parents = [FibreTarget]
+    parents = [OneOf(L1Spectrum, one2one=True)]
+    children = [Optional('self', idname='adjunct', one2one=True)]
+    identifier_builder = ['l1_spectrum']
+    indexes = [None]  # removes nspec index
 
 
 class L1SingleSpectrum(L1Spectrum, Single):
@@ -78,6 +81,7 @@ class L1SupertargetSpectrum(L1StackedSpectrum, Supertarget):
     factors = ['id']
     parents = [Multiple(L1SingleSpectrum, 2, 10, constrain=(WeaveTarget, ArmConfig))] + L1Spectrum.parents
     identifier_builder = ['l1single_spectra', 'id']
+    indexes = []
 
 
 
